.equ opwr 			= 13						;dBM
.equ FREQUENCY		= 868330000
.equ BITRATE		= 12000
.equ XTALFREQ		= 32000000
.equ MAX_ARRAYSIZE	= 20

.equ FRF			= ((FREQUENCY*524288 + (XTALFREQ/2)) / XTALFREQ)
.equ FRF_MSB		= ((FRF>>16) & 0xFF)
.equ FRF_MID		= ((FRF>>8) & 0xFF)
.equ FRF_LSB		= ((FRF) & 0xFF)

.equ BR_VAL			= (((((( 344828 - (BITRATE<2694)*301725 ) + (BITRATE/2))/BITRATE) - 1) | ( (BITRATE<2694) * 0x80 )))
.if BR_VAL<128
.equ BR				= ((10000000 + (29*BR_VAL/2))/(29*(BR_VAL+1)))
.endif

.equ DATARATE		= ((XTALFREQ + (BR/2)) / BR)
.equ DATARATE_MSB	= (DATARATE>>8)
.equ DATARATE_LSB	= (DATARATE & 0xFF)

;Bitrate
.equ bw				= 2
.equ freqdev		= 737

;------ rf69_data -------------------------------------------------------------
;
;	Sender: Daten werden mit data bertragen und im FIFO gespeichert,
;	Empfnger: Gespeicherte Daten werden in data geschoben.
;
;-------------------------------------------------------------------------------
rf69_data:
	ldi		temp, 8				;8 Datenbits 

rf69_data_loop:
	lsl		data				;Schieben und MSB ins Carry
	cbi		RF_PORT, SDI		;SDI auf 0
	brcc	rf69_data_0			;Wenn Carry schon leer ist, so lassen
	sbi		RF_PORT, SDI		;Andernfalls SDI auf 1

rf69_data_0:
	sbic	RF_PIN, SDO			;Wenn SDO auf 1
	ori		data, 1				;Ins derzeitige LSB 1 schreiben
 
	sbi		RF_PORT, SCK		;Clock-Impuls erzeugen
	nop					
	nop
	cbi		RF_PORT, SCK

	dec		temp				;Bit Zhler runterzhlen
	brne	rf69_data_loop		;Wenn nicht abgelaufen, wiederholen

	cbi		RF_PORT, SDI		;SDI auf 0
ret

;***************************************************************************************
RFM_INIT:
	sbi		RF_DDR, SDI
	sbi		RF_DDR, SCK
	ldiw_w	Z, RFM_CONFIG_TBL	;Pointer auf Register
RFM_init_l0:
	lpm		xh, z+
	lpm		xl, z+
	cpi		xh, 0xFF			;Ende der RFM_CONFIG_TBL
	breq	RFM_init_end
	rcall	RFM_SEND_CMD_w
	rjmp	RFM_init_l0
RFM_init_end:
	rcall	wait
ret

;**************************************************************************************
RFM_SEND_CMD_w:	
	sbr		xh,(1<<7)			;in Register schreiben
RFM_SEND_CMD:					;Register auslesen
	cbi		RF_PORT, CS
	mov		data, xh			;X=CMD 16Bit
	rcall	rf69_data
	mov		data, xl
	rcall	rf69_data
		;U1=data
	sbi		RF_PORT, CS
ret


;****************************************************************************************
rfm_rxon:
	ldiw	x, 0x0110			;Receiver mode
	rcall	RFM_SEND_CMD_w
rfm_rxon_l0:
	ldiw	x, 0x27FF			;RX on (set to receiver mode in RegOpMode)
	rcall	RFM_SEND_CMD
	andi	data, (1<<7)		;wait for Mode-Change-Flag
	breq	rfm_rxon_l0
rfm_rxon_l1:
	ldiw	x, 0x27FF			;Wait for RX-Ready-Flag
	rcall	RFM_SEND_CMD
	andi	data, (1<<6)
	breq	rfm_rxon_l1
rfm_rxon_l2:
	ldiw	X, 0x28FF			;Wait for PayloadReady-Flag
	rcall	RFM_SEND_CMD
	andi	data, (1<<2)
	breq	rfm_rxon_l2			
ret

;****************************************************************************************
RFM_transmit:					
	rcall	RFM_stb				;Go To Standby und clear FIFO
		;// Write data to FIFO
	cbi		RF_PORT, CS
	ldi		data, 0x80 + RegFifo
	rcall	rf69_data
	ldiw	Z, fifoarray
	ld		temp1, Z+			;Packet Lnge
	mov		data, temp1
RFM_transmit_l0:
	rcall	rf69_data			
	ld		data, Z+			;nchstes byte
	subi	temp1, 1
	brcc	rfm_transmit_l0		;alle bytes im FIFO?
	sbi		RF_PORT, CS
	
	rcall	rfm_txon			;FIFO senden
	;rcall	wait_10ms
	;rcall	FIFO_read
RFM_transmit_l1:
	ldiw	X, 0x28FF
	rcall	RFM_SEND_CMD
	andi	data, (1<<3)
	breq	rfm_transmit_l1		;Wait for PacketSent-Flag
	rcall	RFM_txoff
ret

;**************************************************************************************
RFM_txon:
	ldiw	X, 0x010C
	rcall	RFM_SEND_CMD_w		;TX on (set to transmitter mode in RegOpMode)
RFM_txon_l0:
	ldiw	X, 0x27FF
	rcall	RFM_SEND_CMD
	andi	data, (1<<7)
	breq	rfm_txon_l0			;Wait for Mode-Ready-Flag
;ret
RFM_txon_l1:
	ldiw	X, 0x27FF
	rcall	RFM_SEND_CMD
	andi	data, (1<<5)
	breq	rfm_txon_l1			;Wait for TXReady-Flag
ret

;***************************************************************************************
RFM_txoff:
	ldiw	X, 0x0108
	rcall	RFM_SEND_CMD_w		;TX off (set to standby mode in RegOpMode)
RFM_txoff_l0:
	ldiw	X, 0x27FF
	rcall	RFM_SEND_CMD
	andi	data, (1<<7)
	breq	rfm_txoff_l0		;Wait for Mode-Ready-Flag
ret

;******************************************************************************************
RFM_stb:
	ldiw	X, 0x0104			;Go To Standby
	rcall	RFM_SEND_CMD_w		;TX/RX off (set to standby mode in RegOpMode)
RFM_rxoff_l0:
	ldiw	X, 0x27FF
	rcall	RFM_SEND_CMD
	andi	data, (1<<7)
	breq	rfm_rxoff_l0		;Wait for Mode-Ready-Flag
	rcall	RFM_fifo_clear		;FIFO lschen
ret

;**************************************************************************************
RFM_sleep:
	ldiw	X, 0x0100
	rcall	RFM_SEND_CMD_w		;TX off (set to sleep mode in RegOpMode)
RFM_sleep_l0:
	ldiw	X, 0x27FF
	rcall	RFM_SEND_CMD
	andi	temp, (1<<7)
	breq	RFM_sleep_l0		;Wait for Mode-Ready-Flag
ret

;****************************************************************************************
RFM_fifo_clear:
	ldiw	X, 0x2810
	rcall	RFM_SEND_CMD_w		;FIFO lschen
ret

;*************RF69_status**********************************************************************
RF69_status:
	ldi		temp, 'S'
	rcall	uart_transmit
	ldi		temp, ':'
	rcall	uart_transmit

	clr		temp2
RF69_status_loop:	
	mov		temp, temp2
	rcall	usart_writehex
	ldi		temp, '='
	rcall	uart_transmit
	mov		XH, temp2
	ldi		XL, 0xFF
	rcall	RFM_SEND_CMD
	mov		temp, data
	push	temp2
	rcall	usart_writebin
	pop		temp2
	ldi		temp, ','
	rcall	uart_transmit
	rcall	usart_space
	inc		temp2
	cpi		temp2, 85			;Register 0x00-0x55 auslesen
	brne	RF69_status_loop	
ret

;*************Register lesen**************************************************************

Opmode:
	ldiw	X, 0x01FF			;RegOpMode auslesen
	mov		temp, XH
	rcall	usart_writehex
	ldi		temp, '='
	rcall	uart_transmit
	rcall	RFM_SEND_CMD
	mov		temp, data
	rcall	usart_writebin
	rcall	usart_space
ret

IrqFlags1:	
	ldiw	X, 0x27FF			;RegIrqFlags1 auslesen
	mov		temp, XH
	rcall	usart_writehex
	ldi		temp, '='
	rcall	uart_transmit
	rcall	RFM_SEND_CMD
	mov		temp, data
	rcall	usart_writebin
	rcall	usart_space
ret

IrqFlags2:	
	ldiw	X, 0x28FF			;RegIrqFlags2 auslesen
	mov		temp, XH
	rcall	usart_writehex
	ldi		temp, '='
	rcall	uart_transmit
	rcall	RFM_SEND_CMD
	mov		temp, data
	rcall	usart_writebin
	ldi		temp, ','
	rcall	uart_transmit
	rcall	usart_space
ret

;*************FIFO auslesen************************************************

FIFO_read:
	ldi		temp, 'F'
	rcall	uart_transmit
	ldi		temp, ':'
	rcall	uart_transmit

	clr		temp2
	cbi		RF_PORT, CS			;CS auf 0(aktiv)
	ldi		data, 0x00			;aus FIFO lesen
	rcall	rf69_data			
FIFO_read_loop:
	ldi		data, 0xFF	
	rcall	rf69_data			;Byte holen und in data speichern
	st		Y+, data			;eingegangenes Byte speichern
	mov		temp, data
	rcall	uart_transmit
	inc		temp2
	cpi		temp2, 10			;FIFO Byte 0x00-0x09 auslesen
	brne	FIFO_read_loop
	sbi		RF_PORT, CS			;CS auf 1(inaktiv)
	rcall	usart_space
ret

;******************************************************************************************
RFM_CONFIG_TBL:	
	.db	RegDataModul, 00				; // FSK, Packet mode, No shaping
		;//Bitrate + corresponding settings (Receiver bandwidth, frequency deviation)
		;//Frequency Deviation
	.db	RegFdevMsb, (freqdev >> 8)
	.db	RegFdevLsb, (freqdev & 0xFF)
		;//Data Rate
	.db	RegBitrateMsb, DATARATE_MSB
	.db	RegBitrateLsb, DATARATE_LSB
		;//Receiver Bandwidth
	.db	RegRxBw,80 | bw
	.db	RegOcp, 0x1B					;// OCP enabled, 100mA
		;// DIO-Mapping
	.db	RegDioMapping1, (2<<DIO2Mapping)|(3<<DIO1Mapping)|(0<<DIO0Mapping)	;// Clkout, FifoFull, FifoNotEmpty, FifoLevel, PacketSent/CrcOk
	.db	RegDioMapping2, 07				;// Clock-Out off
		;// Carrier frequency
	.db	RegFrfMsb, FRF_MSB
	.db	RegFrfMid, FRF_MID
	.db	RegFrfLsb, FRF_LSB
		;// Packet config
	.db	RegPacketConfig1, 0x00			;// Variable length, No DC-free encoding/decoding, CRC-Check, No Address filter
	.db	RegPayloadLength, 0x0B	;// Max. Payload-Length
	.db	RegFifoThresh, 0x80				;// Tx-Start-Condition: FIFO not empty
	.db	RegPacketConfig2, 0x12			;// Packet-Config2
	.db	RegNodeAdrs, 0xF2
		;// Prambel length 3 bytes
	.db	RegPreambleMsb, 0x00
	.db	RegPreambleLsb, 0x03
		;// Sync-Mode
	.db	RegSyncConfig, 0x88				;// set FIFO mode
	.db	RegSyncValue1, 0xE5				;// sync word MSB
	.db	RegSyncValue2, 0x8C				;// sync word LSB
		;// Receiver config
	.db	RegLna, 0x00					;// LNA: 50 Ohm Input Impedance, Automatic Gain Control
	.db	RegTestLna, 0x2D				;// High sensitivity mode
	.db	RegTestDagc, 0x30				;// Improved DAGC
	.db	RegRssiThresh, 0xDC				;// RSSI mind. -110 dBm
	.db	RegAfcFei, 0x2D					;// Start AFC, Auto-On
	.db	RegPaLevel, 0b10000000+(opwr+18)	;// Set Output Power	
	.db	0xFF, 0xFF
