;       		    ATtiny2313
;   				+--+-+--+                        
;	    (/RESET)PA2	|  |_|	| VCC                       
;   	   (RXD)PD0	|		| PB7(SCL)                    
;   	   (TXD)PD1	|		| PB6(MISO)               
;	     (XTAL2)PA1 |       | PB5(MOSI) 	
;	     (XTAL1)PA0 |       | PB4(OC1B) 	
;         (INT0)PD2 |       | PB3(OC1A) 	
;	      (INT0)PD3 |       | PB2(OC0A)
;   	    (T0)PD4 |       | PB1(AIN1)
;           (T1)PD5 |		| PB0(AIN0)            
;               GND |		| PD6            
;   				+-------+                           
; 
; PA0	I	XTAL1, RFM12-CLK
; PA1	I	XTAL2
; PA2	I	RESET
; PB0	O	RFM12-CS 
; PB1	O	RFM12-SCK
; PB2	O	RFM12-SDI 
; PB3	I	RFM12_SDO
; PB4	I	Taster 1
; PB5	I	MOSI, Taster 2
; PB6	I	MISO
; PB7	I	SCK  
; PD0	I	RXD
; PD1	O	TXD 
; PD2	I	nc (pullup on) 
; PD3	I	nc (pullup on) 
; PD4	I	nc (pullup on)    
; PD5	I	nc (pullup on) 
; PD6	O	LED 

.include "tn2313def.inc"

.DSEG
	fifoarray: .BYTE 12

.CSEG
.equ F_CPU		= 8000000

.equ BAUD		= 38400						; Baudrate des UART

;Portbelegung
.EQU RF_PORT	= PORTB
.EQU RF_DDR		= DDRB
.EQU RF_PIN		= PINB

.EQU CS			= 0
.EQU SCK		= 1
.EQU SDI		= 2
.EQU SDO		= 3

.EQU AUX_PORT	= PORTD
.EQU AUX_DDR	= DDRD
.EQU AUX_PIN	= PIND

.EQU LED		= 6

.def temp			= r16
.def temp1			= r17
.def temp2			= r18
.def data			= r19
.def cnt			= r20
.def datacnt		= r21
.def StatusA		= r22

.macro  ldiw						;16-Bit Adresse aus Sram laden
  ldi	@0L,low(@1)
  ldi	@0H,high(@1)
.endmacro

.macro  ldiw_w						;16-Bit Adresse aus Flash laden
  ldi	@0L,low(@1*2)
  ldi	@0H,high(@1*2)

.endmacro

.cseg 

;Init
start:
    ldi     temp, RAMEND
    out     SPL, temp
 
	ldi		temp, 0xFF
	out		PORTD, temp				;alle Ports mit pullup
 	
	ldi		temp, (1<<SDI)|(1<<SCK)|(1<<CS)	;Ports als Ausgang
	out		RF_DDR, temp
	ldi		temp, (1<<CS)			;CS auf high, not selected
	out		RF_PORT, temp
	
	ldi		temp, (1<<LED)
	out		AUX_DDR, temp
	
;beim Start ...
	sbi		AUX_PORT, LED
	rcall	UART_init
	rcall	RFM_INIT
	rcall	UART_Initial
	;rcall	RF69_status
	cbi		AUX_PORT, LED

	;rjmp	RFM_RX

RFM_TX:
	sbi		AUX_PORT, LED
	rcall	RFM_senden_1
	rcall	wait_1s
	rcall	RFM_senden_2
	cbi		AUX_PORT, LED
	rcall	wait_500ms
rjmp	RFM_TX

RFM_RX:
	sbi		AUX_PORT, LED
	rcall	RFM_empfangen
	cbi		AUX_PORT, LED
	rcall	wait_500ms
rjmp	RFM_RX

;*************************************************************************************
RFM_empfangen:	
	rcall	RFM_rxon
		;FIFO auslesen
	ldiw	Z, fifoarray
	ldi		temp2, MAX_ARRAYSIZE
	mov		data, xh			;X=CMD 16Bit
	cbi		RF_PORT, CS
	ldi		data, 0x00				;aus FIFO lesen
	rcall	rf69_data
	st		Z+, data
	mov		temp1, data				;Anzahl
getFiFo_l0:
	ldi		data, 0xFF	
	rcall	rf69_data				;Byte holen und in data speichern
	tst		temp2
	breq	getFiFo_m0
	mov		temp, data
	rcall	uart_transmit
	dec		temp2
	dec		temp1
	brne	getFiFo_l0
getFiFo_m0:	
	sbi		RF_PORT, CS
	rcall	RFM_stb
ret

;*************************************************************************************
RFM_senden_1:
	rcall	UART_Aktion_1
	ldiw_w	Z, Befehl_0
	ldiw	Y, fifoarray
	rjmp	senden

RFM_senden_2:
	rcall	UART_Aktion_2
	ldiw_w	Z, Befehl_1
	ldiw	Y, fifoarray
senden:						;Daten in fifoarray ablegen, 1.Byte = Anzahl der Nutzbytes
	lpm		temp, Z+
	tst		temp
	breq	_senden
	st		Y+, temp
	rjmp	senden
_senden:	
	rcall	RFM_transmit
ret

;**************************************************************************************
Befehl_0:
	.db 9,242,"SXXAXXA",0
Befehl_1:		
 	.db	9,242,"SYYBYYB",0

_Aktion_1:
	.db "Auftrag 1 ausfuehren!!!", 10, 13,0

_Aktion_2:
	.db "Auftrag 2 ausfuehren!!!", 10, 13,0

Initial:
	.db "Empfaenger Init erfolgreich!!",10,13,0

.include "RFM69.inc"
.include "RFM69_reg.inc"
.include "UART.inc"
.include "Hilfs.inc"
